/*
 * Copyright (C) 2005  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef AVRDEBUG_H
#define AVRDEBUG_H

#include <string>
#include <vector>

#include "debugcmd.h"

typedef std::vector<unsigned char> ResultBlock;

/**
 * Klasse, die eine Debugschnittstelle zu einem Controller bereitstellt.
 * Hier gibt es nur allgemeine Dinge; von dieser Klasse müssen also noch
 * die Klassen abgeleitet werden, die die wirkliche Kommunikation zur 
 * Verfügung stellen.
 */
class avrdebug
{
  public:
   
   /**
    * Is everything OK on this interface?
    */
   bool isOK();
   
   /**
    * 
    */
   std::string getErr();

   /**
    * Returns true if the command cmd has been added to the command block.
    * The command block is full otherwise and has to be executed.
    * 
    * The default implementation does only allow one command in a block.
    */
   virtual bool addCmd(DebugCmd cmd);
   
   /**
    * Executes the command block. The default implementation executes a block 
    * as single commands.
    */
   virtual void executeCmdBlock(ResultBlock& results);
   
   /**
    * clears the command block
    */
   virtual void clearCmdBlock();
   
   virtual void disconnect() {};
   
   virtual int  connect() { return(0); };
   
   virtual int  resetTarget() { return(-1); };
   
   virtual int  setVoltages(float u_setpoint, float u_max) { return(-1); };
   
   virtual ~avrdebug() {};
   
   bool executed() { return(fExecuted); };
   
  protected:
   std::string            firstErr;
   std::vector<DebugCmd>  cmdblock;
   
   /**
    * Indicates whether the command block has been executed after
    * its last change.
    */
   bool fExecuted;
   
   virtual void executeCmd(DebugCmd cmd, ResultBlock& results) = 0;
   
   void setErr(std::string errstring);
   void setOK();
    
  private:
   bool fOK;   
};

#endif
