/*
 * Copyright (C) 2005-2007  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef AVRDEBUG_SER_H
# define AVRDEBUG_SER_H

# include "avrdebug.h"
# include "serial_posix.h"


/**
 * Klasse, die eine Debugschnittstelle zu einem Controller bereitstellt.
 *
 * Hier wird die serielle Schnittstelle benutzt.
 * 
 * Dieses Protokoll unterstützt ein paar Spezialitäten, die die anderen Protokolle nicht
 * kennen. Diese Features lassen sich aber auch nicht gut verallgemeinern.
 * 
 */
class avrdebug_ser : public avrdebug
{
public:
  avrdebug_ser(const char* device,
               int         baudrate,
               std::string specialopts = "");
  ~avrdebug_ser();
  
  void executeCmd(DebugCmd cmd, ResultBlock& results);   
  
  /**
   * Spezielles Feature dieses Protokolls
   * 
   * Liefert ungleich null zurück, wenn das nicht erfolgreich war.
   */
  int setReportOnWrite(bool fVal);
  
  /**
   * Spezielles Feature dieses Protokolls
   */
  void setAdr_flags(int nVal) { nAdrFlags = nVal; };
  
  /**
   * Spezielles Feature dieses Protokolls
   */
  void setAdr_errcnt(int nVal) { nAdrErrcnt = nVal; };
  
  /**
   * Spezielles Feature dieses Protokolls
   */
  int clearErrCnt();
  
  /**
   * Spezielles Feature dieses Protokolls
   */
  int getErrCnt();
  
private:
  Serial* serial;
  unsigned char uchCRCReg;
  
  /**
   * Soll es beim Schreiben die Information über den vorigen Wert geben?
   */
  bool fReportOnWrite;
  
  /**
   * Adresse des Bytes mit den Flags
   */
  int nAdrFlags;
  
  /**
   * Adresse des Bytes mit dem Fehlerzähler
   */
  int nAdrErrcnt;
  
  void crc_byte(unsigned char uchData);
  void checkSerial();
};

#endif
