/*
 * Copyright (C) 2005-2007  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef AVRDEBUG_SER2_H
# define AVRDEBUG_SER2_H

# include "avrdebug.h"
# include "serial.h"

/**
 * Klasse, die eine Debugschnittstelle zu einem Controller bereitstellt.
 *
 * Hier wird die serielle Schnittstelle benutzt.
 */
class avrdebug_ser2 : public avrdebug
{
public:
  
  enum eTargetCon { eTARGET_SPI, eTARGET_UART, eTARGET_SELF };
  
  /**
   * @param targetBaudrate: baudrate (UART), clock in kHz (SPI)
   */
  avrdebug_ser2(const char* device,
                int         baudrate,
                eTargetCon  targetCon,
                int         targetBaudrate,
                std::string specialopts = "");
  
  void executeCmd(DebugCmd cmd, ResultBlock& results);
  
  virtual ~avrdebug_ser2();
  
  /**
   * Overwrites the default implementation to allow for optimization.
   */
  bool addCmd(DebugCmd cmd);
  
  /**
   * Executes the command block. This implementation may use optimization.
   */
  void executeCmdBlock(ResultBlock& results);
  
  /**
   * clears the command block
   */
  void clearCmdBlock();
  
  int  setVoltages(float u_setpoint, float u_max);
  
  int  resetTarget();
  
  int  connect();
  
  void disconnect();
  
private:
  Serial*           serial;
  unsigned char     uchCRCReg;
  
  /**
   * How many commands can be grouped into one block?
   */
  unsigned int      uMaxCmdsInInternalBlock;
  
  /**
   * How many commands are in the current block?
   */
  unsigned int      uCmdsInExternalBlock;
  
  eTargetCon  targetCon;
  int         targetBaudrate;
  
  int  sync();
  
  /**
   * Liefert Null zurück, wenn avrdev das Kommando angenommen hat oder wenn (nCheck <= 2) ist.
   * Wenn (nCheck > 2) ist und avrdev das Kommando nicht bestätigt hat, wird -1 geliefert.  
   */
  int writeCmd(unsigned char b0, 
               unsigned char b1, 
               unsigned char b2, 
               unsigned char b3,
               int           nCheck = 0);
  
  /**
   * How many internal commands does cmd equal?
   */
  unsigned int nrOfInternalCmds(DebugCmd cmd);
  
  
  /**
   * Executes the block of commands given in cmds. This block must be suitable
   * for block execution using CMD_debug_waitbytes. It must only contain commands
   * which can be mapped to one (and only one) low-level command.
   */
  void executeBlock(std::vector<DebugCmd> cmds,
                    ResultBlock&          results);
  
  /**
   * Returns zero on success
   */
  int connect(eTargetCon  targetCon,
              int         targetBaudrate);
    
  int  checkErr(int nVal, int nCmd = -1);
  
  void crc_byte(unsigned char uchData);
  
  void checkSerial();
};

#endif
