/*
 * Copyright (C) 2007  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "serial.h"

#include "serial_posix.h"

#ifdef HASLIBFTDI
# include "serial_ftdi.h"
#endif

#ifdef LINUXFTDILL
# include "serial_ftdilinuxll.h"
#endif

// 0: keine Ausgaben
// 1: nur Fehler
// 2: weiteres
#define DEBUG 0

# include <stdio.h>

Serial::Serial()
{
#if DEBUG > 1
  printf("Serial::Serial()\n");
#endif
  firsterr = "";
}

Serial::~Serial()
{
  if (!IsOK())
    printf("Serial::~Serial(): %s\n", GetFirstErr().c_str());
}

void Serial::SetErr(std::string newerr)
{
  if (IsOK())  
    firsterr = newerr;
#if DEBUG > 0
  printf("Serial::SetErr(%s)\n", newerr.c_str());
#endif
}

Serial* Serial::CreateSerial(const char* sd, int baudrate, std::string specialopts)
{
  Serial* serial = 0;
  
#ifdef HASLIBFTDI
  if (serial == 0 && specialopts.find(",ttyS_ftdi,") != std::string::npos)
  {
    std::string descrstr;
    std::string serialstr;
    const char* descrp;
    const char* serialp;
    std::string::size_type pos1;
    
    pos1 = specialopts.find(",ttyS_ftdi_descr=");
    if (pos1 == std::string::npos)
      descrp = 0;
    else
    {
      descrstr = specialopts.substr(pos1+17, specialopts.find(",", pos1+17)-pos1-17);
      descrp   = descrstr.c_str();
    }
    pos1 = specialopts.find(",ttyS_ftdi_serial=");
    if (pos1 == std::string::npos)
      serialp = 0;
    else
    {
      serialstr = specialopts.substr(pos1+18, specialopts.find(",", pos1+18)-pos1-18);
      serialp   = serialstr.c_str();
    }

    serial = new SerialFTDI(descrp, serialp, baudrate, specialopts);
  }
#endif
  
#ifdef LINUXFTDILL
  if (serial == 0 && specialopts.find(",ftdi_linux_ll,") != std::string::npos)
  {
    // LibUSB initialisation:
    usb_init();
    //
    serial = new SerialFTDILinuxLL(specialopts);
  }
#endif
  
  if (serial == 0)
    serial = new SerialPOSIX(sd, baudrate);
  
  return(serial);
}

int Serial::ReadBytes(unsigned char* databuf, int nMaxBytesToRead)
{
  return(ReadByte(*databuf));
}

