/*
 * Copyright (C) 2007  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef SERIAL_H
# define SERIAL_H


# include <string>

/**
 * 
 */
class Serial
{
public:

  Serial();
  virtual ~Serial();
    
  /**
   * Writes a single byte to the serial device.
   */
  virtual void WriteByte(unsigned char uchVal) = 0;
  
  /**
   * Writes nBufLen bytes, returns number of bytes written.
   */
  virtual int WriteBytes(const unsigned char* buf,
                         int                  nBufLen) = 0;

  /**
   * On error, returns -1.
   * nTimeout in µs.
   */
  virtual int ReadByte(bool fSetErr  = true,
                       int  nTimeout = 200000) = 0;
  
  /**
   * Just tries to read a byte -- if there is nothing to read, it
   * returns 0, otherwise 1.
   * On error, returns -1.
   */
  virtual int ReadByte(unsigned char& data) = 0;
  
  /**
   * Read as many bytes as available instantly. Return number of bytes
   * read or a negative value on error.
   */
  virtual int ReadBytes(unsigned char* databuf, int nMaxBytesToRead);

  /**
   * Discards data from input/output.
   */
  virtual void Flush() = 0;
  
  /**
   * Waits until everything is really written to the device. 
   * Might not work, so better don't rely on it.
   */
  virtual void WaitUntilWritten() = 0;
  
  /**
   * Changes baudrate. If you want to ensure everything is really written out
   * to the device before, call WaitUntilWritten.
   */
  virtual void ChangeBaudrate(int baudrate) = 0;
  
  
  bool IsOK() { return(firsterr.length() == 0); };
  
  std::string GetFirstErr() { return(firsterr); };
  
  int GetBaudrate() { return(nBaudrate); };  
  
  /**
   * Usually returns a SerialPOSIX.
   * 
   * With HASLIBFTDI defined and specialopts containing ",ttyS_ftdi,", a SerialFTDI
   * is created. Optional settings in specialopts:
   *     ttyS_ftdi_descr      USB descriptor string           
   *     ttyS_ftdi_serial     USB serial string
   * 
   * With LINUXFTDILL defined and specialopts containing ",ftdi_linux_ll," a
   * SerialFTDILinuxLL is created.
   */
  static Serial* CreateSerial(const char* sd, int baudrate, std::string specialopts);
  
  /**
   * Wait until data arrives, but timeout after nTimeMax_ms in any case.
   * The default implementation returns immediately.
   * Should return non-zero if data has arrived.
   */
  virtual int WaitForData(int nTimeMax_ms) { return(0); };
  
protected:
  void SetErr(std::string newerr);
    
  std::string firsterr;
  int         nBaudrate;
};

#endif
