/*
 * Copyright (C) 2007  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef SERIAL_FTDI_H
# define SERIAL_FTDI_H

# include "serial.h"

# include <ftdi.h>
# include <string>

/**
 * This class depends on libftdi, which depends on libusb.
 * 
 * I did not have documentation on libftdi, but looked at the source code (ftdi.h and ftdi.c).
 * 
 * Old version of libftdi/libusb in /usr/lib or /lib, newer version in /usr/local/lib and error
 * "./dlf_recorder: relocation error: ./dlf_recorder: undefined symbol: ftdi_set_interface":
 *  - see man ld.so
 *  - ldconfig not used after installing libraries?
 * 
 * Kernel modules ftdi_sio and usbserial have to be removed (rmmod).
 */
class SerialFTDI : public Serial
{
public:
  
  /**
   * Opens serial port, discards data from input/output.
   * Program has to be run as root to use this constructor.
   * 
   * Fixed settings:
   *   vendor_id =0x0403
   * Defaults:
   *   product_id=0x6001
   * 
   * Specialopts: 
   *    ftdi_ReadChunksize=size
   *    ftdi_Latency=LatencyInMilliSeconds
   *    ftdi_PID=0xHHHH
   */
  SerialFTDI(const char*  usb_descr_string,
             const char*  usb_serial_string,
             int          baudrate,
             std::string  specialopts = "");
  
  /**
   * Closes serial port. Does not wait until data is written!
   */
  virtual ~SerialFTDI();
  
  /**
   * Writes a single byte to the serial device.
   */
  virtual void WriteByte(unsigned char uchVal);
  
  /**
   * Writes nBufLen bytes, returns number of bytes written.
   */
  virtual int WriteBytes(const unsigned char* buf,
                         int                  nBufLen);
  
  /**
   * On error, returns -1.
   * nTimeout in µs.
   */
  virtual int ReadByte(bool fSetErr  = true,
                       int  nTimeout = 200000);
  
  /**
   * Just tries to read a byte -- if there is nothing to read, it
   * returns 0, otherwise 1.
   * On error, returns -1.
   */
  virtual int ReadByte(unsigned char& data);
  
  /**
   * Read as many bytes as available instantly. Return number of bytes
   * read or a negative value on error.
   */
  virtual int ReadBytes(unsigned char* databuf, int nMaxBytesToRead);

  /**
   * Discards data from input/output.
   */
  virtual void Flush();
  
  /**
   * Sadly, this doesn't work...
   */
  virtual void WaitUntilWritten();
    
  /**
   * Changes baudrate. If you want to ensure everything is really written out
   * to the device before, call WaitUntilWritten.
   */
  virtual void ChangeBaudrate(int baudrate);
  
  /**
   *  
   */
  void SetReadChunkSize(unsigned int chunksize);
  
private:
  struct ftdi_context  ftdi;
};

#endif
