/*
 * Copyright (C) 2007  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef SERIAL_POSIX_H
# define SERIAL_POSIX_H

#include "serial.h"

# ifndef WIN32
#  include <termios.h> // seriell
# endif

# include <string>

/**
 * 
 */
class SerialPOSIX : public Serial
{
public:

  /**
   * Opens serial port, discards data from input/output.
   */
  SerialPOSIX(const char*  sd,
         int          baudrate);
  
  /**
   * Closes serial port. Does not wait until data is written!
   */
  virtual ~SerialPOSIX();
  
  /**
   * Writes a single byte to the serial device.
   */
  virtual void WriteByte(unsigned char uchVal);
  
  /**
   * Writes nBufLen bytes, returns number of bytes written.
   */
  virtual int WriteBytes(const unsigned char* buf,
                         int                  nBufLen);
  
  /**
   * On error, returns -1.
   * nTimeout in µs.
   */
  virtual int ReadByte(bool fSetErr  = true,
                       int  nTimeout = 200000);
  
  /**
   * Just tries to read a byte -- if there is nothing to read, it
   * returns 0, otherwise 1.
   * On error, returns -1.
   */
  virtual int ReadByte(unsigned char& data);
  
  /**
   * Discards data from input/output.
   */
  virtual void Flush();
  
  /**
   * Waits until everything is really written to the device
   */
  virtual void WaitUntilWritten();
  
  /**
   * Changes baudrate. If you want to ensure everything is really written out
   * to the device before, call WaitUntilWritten.
   */
  virtual void ChangeBaudrate(int baudrate);
 
private:
  
  void Set_input_mode(int baudrate);
  void Reset_input_mode();
 
# ifndef WIN32
  int            serPort;          // der file-descriptor
  struct termios saved_attributes; // Use this variable to remember original terminal attributes.
  struct termios tattr;
# endif
};

#endif
