/*
 * Copyright (C) 2005-2007  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef USRCMD_H
#define USRCMD_H

#include <string>
#include <exception>
#include "uservars.h"

class UserCmd
{
  public:
   
   enum eRelation  { eEQUAL, eNOTEQUAL, eGREATER, eSMALLER };   
   enum eCmd       { eRW, eQUIT, eWAIT, eEOF, eLOOP, eREADFILE, eNEWLINE, eNOP, eHELP, eECHO, eSLEEP };
   enum eDisplay   { eSIGNED, eUNSIGNED, eHEX, eALL };
   enum eShowValue { eNORMAL, eQUIET, eREALLY_QUIET };
  
   static void SetVarCache(UserVars* vc) { vars = vc; };
  
   /**
    * Returns non-zero if the command contains any errors
    */
   int setCmd(std::string cmdstr);
   
   /**
    * 
    */
   std::string explainCmd() { return(explanation); };
   
   std::string getErr()     { return(error);       };
   
   std::string getEcho()    { return(echo);        };      
   
   bool isOK()              { return(!fErr);       };
   
   bool write()   { return(fWrite);    };
   int  width()   { return(nWidth);    };
   int  adr()     { return(nAdr);      };
   int  size()    { return(nSize);     };
   int  value()   { return(nVal);      };
   
   std::string getFilename() { return(filename); };

   eDisplay   display()      { return(myDisplay);  };   
   eShowValue showValue()    { return(myShowValue);};
   eCmd       cmd()          { return(myCmd);      };   
   eRelation  relation()     { return(myRelation); };

   void setShowValue(eShowValue newShowValue) { myShowValue = newShowValue; };
  
   void WriteValueToVarCache(int val);
  
  private:
  
   static UserVars*   vars;
  
   eDisplay    myDisplay;
   eShowValue  myShowValue;
   eRelation   myRelation;
   eCmd        myCmd;
   bool        fWrite;
   int         nWidth; // Number of bits per value
   int         nAdr;
   int         nVal;
   int         nSize;  // Number of bytes to read
   bool        fErr;
   std::string myVarname;
   std::string error;
   std::string filename;
   std::string explanation;
   std::string echo;
   
   void setErr(std::string descr);
   std::string GetVarName(std::string in, std::string::size_type& pos);
};

#endif
